@@ -1,14 +1,15 @@
 """Support for Timers."""
 from __future__ import annotations
 
+from collections.abc import Callable
 from datetime import datetime, timedelta
 import logging
-from typing import Callable
 
 import voluptuous as vol
 
 from homeassistant.const import (
     ATTR_EDITABLE,
+    ATTR_ENTITY_ID,
     CONF_ICON,
     CONF_ID,
     CONF_NAME,
@@ -31,10 +32,16 @@ DOMAIN = "timer"
 ENTITY_ID_FORMAT = DOMAIN + ".{}"
 
 DEFAULT_DURATION = 0
+DEFAULT_RESTORE = False
+
 ATTR_DURATION = "duration"
 ATTR_REMAINING = "remaining"
 ATTR_FINISHES_AT = "finishes_at"
+ATTR_RESTORE = "restore"
+ATTR_FINISHED_AT = "finished_at"
+
 CONF_DURATION = "duration"
+CONF_RESTORE = "restore"
 
 STATUS_IDLE = "idle"
 STATUS_ACTIVE = "active"
@@ -54,16 +61,11 @@ SERVICE_FINISH = "finish"
 STORAGE_KEY = DOMAIN
 STORAGE_VERSION = 1
 
-CREATE_FIELDS = {
-    vol.Required(CONF_NAME): vol.All(str, vol.Length(min=1)),
-    vol.Optional(CONF_NAME): cv.string,
+STORAGE_FIELDS = {
+    vol.Required(CONF_NAME): cv.string,
     vol.Optional(CONF_ICON): cv.icon,
     vol.Optional(CONF_DURATION, default=DEFAULT_DURATION): cv.time_period,
-}
-UPDATE_FIELDS = {
-    vol.Optional(CONF_NAME): cv.string,
-    vol.Optional(CONF_ICON): cv.icon,
-    vol.Optional(CONF_DURATION): cv.time_period,
+    vol.Optional(CONF_RESTORE, default=DEFAULT_RESTORE): cv.boolean,
 }
 
 
@@ -91,6 +93,7 @@ CONFIG_SCHEMA = vol.Schema(
                     vol.Optional(CONF_DURATION, default=DEFAULT_DURATION): vol.All(
                         cv.time_period, _format_timedelta
                     ),
+                    vol.Optional(CONF_RESTORE, default=DEFAULT_RESTORE): cv.boolean,
                 },
             )
         )
@@ -128,7 +131,7 @@ async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     await storage_collection.async_load()
 
     collection.StorageCollectionWebsocket(
-        storage_collection, DOMAIN, DOMAIN, CREATE_FIELDS, UPDATE_FIELDS
+        storage_collection, DOMAIN, DOMAIN, STORAGE_FIELDS, STORAGE_FIELDS
     ).async_setup(hass)
 
     async def reload_service_handler(service_call: ServiceCall) -> None:
@@ -162,12 +165,11 @@ async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
 class TimerStorageCollection(collection.StorageCollection):
     """Timer storage based collection."""
 
-    CREATE_SCHEMA = vol.Schema(CREATE_FIELDS)
-    UPDATE_SCHEMA = vol.Schema(UPDATE_FIELDS)
+    CREATE_UPDATE_SCHEMA = vol.Schema(STORAGE_FIELDS)
 
     async def _process_create_data(self, data: dict) -> dict:
         """Validate the config is valid."""
-        data = self.CREATE_SCHEMA(data)
+        data = self.CREATE_UPDATE_SCHEMA(data)
         # make duration JSON serializeable
         data[CONF_DURATION] = _format_timedelta(data[CONF_DURATION])
         return data
@@ -179,7 +181,7 @@ class TimerStorageCollection(collection.StorageCollection):
 
     async def _update_data(self, data: dict, update_data: dict) -> dict:
         """Return a new updated data object."""
-        data = {**data, **self.UPDATE_SCHEMA(update_data)}
+        data = {CONF_ID: data[CONF_ID]} | self.CREATE_UPDATE_SCHEMA(update_data)
         # make duration JSON serializeable
         if CONF_DURATION in update_data:
             data[CONF_DURATION] = _format_timedelta(data[CONF_DURATION])
@@ -198,6 +200,10 @@ class Timer(RestoreEntity):
         self._remaining: timedelta | None = None
         self._end: datetime | None = None
         self._listener: Callable[[], None] | None = None
+        self._restore: bool = self._config.get(CONF_RESTORE, DEFAULT_RESTORE)
+
+        self._attr_should_poll = False
+        self._attr_force_update = True
 
     @classmethod
     def from_yaml(cls, config: dict) -> Timer:
@@ -207,16 +213,6 @@ class Timer(RestoreEntity):
         timer.editable = False
         return timer
 
-    @property
-    def should_poll(self):
-        """If entity should be polled."""
-        return False
-
-    @property
-    def force_update(self) -> bool:
-        """Return True to fix restart issues."""
-        return True
-
     @property
     def name(self):
         """Return name of the timer."""
@@ -243,6 +239,8 @@ class Timer(RestoreEntity):
             attrs[ATTR_FINISHES_AT] = self._end.isoformat()
         if self._remaining is not None:
             attrs[ATTR_REMAINING] = _format_timedelta(self._remaining)
+        if self._restore:
+            attrs[ATTR_RESTORE] = self._restore
 
         return attrs
 
@@ -253,43 +251,62 @@ class Timer(RestoreEntity):
 
     async def async_added_to_hass(self):
         """Call when entity is about to be added to Home Assistant."""
-        # If not None, we got an initial value.
-        if self._state is not None:
+        # If we don't need to restore a previous state or no previous state exists,
+        # start at idle
+        if not self._restore or (state := await self.async_get_last_state()) is None:
+            self._state = STATUS_IDLE
             return
 
-        state = await self.async_get_last_state()
-        self._state = state and state.state == state
+        # Begin restoring state
+        self._state = state.state
+        self._duration = cv.time_period(state.attributes[ATTR_DURATION])
+
+        # Nothing more to do if the timer is idle
+        if self._state == STATUS_IDLE:
+            return
+
+        # If the timer was paused, we restore the remaining time
+        if self._state == STATUS_PAUSED:
+            self._remaining = cv.time_period(state.attributes[ATTR_REMAINING])
+            return
+        # If we get here, the timer must have been active so we need to decide what
+        # to do based on end time and the current time
+        end = cv.datetime(state.attributes[ATTR_FINISHES_AT])
+        # If there is time remaining in the timer, restore the remaining time then
+        # start the timer
+        if (remaining := end - dt_util.utcnow().replace(microsecond=0)) > timedelta(0):
+            self._remaining = remaining
+            self._state = STATUS_PAUSED
+            self.async_start()
+        # If the timer ended before now, finish the timer. The event will indicate
+        # when the timer was expected to fire.
+        else:
+            self._end = end
+            self.async_finish()
 
     @callback
-    def async_start(self, duration: timedelta):
+    def async_start(self, duration: timedelta | None = None):
         """Start a timer."""
         if self._listener:
             self._listener()
             self._listener = None
-        newduration = None
-        if duration:
-            newduration = duration
 
         event = EVENT_TIMER_STARTED
-        if self._state == STATUS_ACTIVE or self._state == STATUS_PAUSED:
+        if self._state in (STATUS_ACTIVE, STATUS_PAUSED):
             event = EVENT_TIMER_RESTARTED
 
         self._state = STATUS_ACTIVE
         start = dt_util.utcnow().replace(microsecond=0)
 
-        if self._remaining and newduration is None:
-            self._end = start + self._remaining
-
-        elif newduration:
-            self._duration = newduration
-            self._remaining = newduration
-            self._end = start + self._duration
-
-        else:
+        # Set remaining to new value if needed
+        if duration:
+            self._remaining = self._duration = duration
+        elif not self._remaining:
             self._remaining = self._duration
-            self._end = start + self._duration
 
-        self.hass.bus.async_fire(event, {"entity_id": self.entity_id})
+        self._end = start + self._remaining
+
+        self.hass.bus.async_fire(event, {ATTR_ENTITY_ID: self.entity_id})
 
         self._listener = async_track_point_in_utc_time(
             self.hass, self._async_finished, self._end
@@ -307,7 +324,7 @@ class Timer(RestoreEntity):
         self._remaining = self._end - dt_util.utcnow().replace(microsecond=0)
         self._state = STATUS_PAUSED
         self._end = None
-        self.hass.bus.async_fire(EVENT_TIMER_PAUSED, {"entity_id": self.entity_id})
+        self.hass.bus.async_fire(EVENT_TIMER_PAUSED, {ATTR_ENTITY_ID: self.entity_id})
         self.async_write_ha_state()
 
     @callback
@@ -319,7 +336,9 @@ class Timer(RestoreEntity):
         self._state = STATUS_IDLE
         self._end = None
         self._remaining = None
-        self.hass.bus.async_fire(EVENT_TIMER_CANCELLED, {"entity_id": self.entity_id})
+        self.hass.bus.async_fire(
+            EVENT_TIMER_CANCELLED, {ATTR_ENTITY_ID: self.entity_id}
+        )
         self.async_write_ha_state()
 
     @callback
@@ -331,10 +350,14 @@ class Timer(RestoreEntity):
         if self._listener:
             self._listener()
             self._listener = None
+        end = self._end
         self._state = STATUS_IDLE
         self._end = None
         self._remaining = None
-        self.hass.bus.async_fire(EVENT_TIMER_FINISHED, {"entity_id": self.entity_id})
+        self.hass.bus.async_fire(
+            EVENT_TIMER_FINISHED,
+            {ATTR_ENTITY_ID: self.entity_id, ATTR_FINISHED_AT: end.isoformat()},
+        )
         self.async_write_ha_state()
 
     @callback
@@ -345,13 +368,18 @@ class Timer(RestoreEntity):
 
         self._listener = None
         self._state = STATUS_IDLE
+        end = self._end
         self._end = None
         self._remaining = None
-        self.hass.bus.async_fire(EVENT_TIMER_FINISHED, {"entity_id": self.entity_id})
+        self.hass.bus.async_fire(
+            EVENT_TIMER_FINISHED,
+            {ATTR_ENTITY_ID: self.entity_id, ATTR_FINISHED_AT: end.isoformat()},
+        )
         self.async_write_ha_state()
 
     async def async_update_config(self, config: dict) -> None:
         """Handle when the config is updated."""
         self._config = config
         self._duration = cv.time_period_str(config[CONF_DURATION])
+        self._restore = config.get(CONF_RESTORE, DEFAULT_RESTORE)
         self.async_write_ha_state()
