@@ -4,6 +4,7 @@ from __future__ import annotations
 import asyncio
 from collections.abc import Callable
 from datetime import datetime, timedelta
+from math import floor
 from typing import Any
 
 from awesomeversion import AwesomeVersion
@@ -11,7 +12,12 @@ from zwave_js_server.client import Client as ZwaveClient
 from zwave_js_server.const import NodeStatus
 from zwave_js_server.exceptions import BaseZwaveJSServerError, FailedZWaveCommand
 from zwave_js_server.model.driver import Driver
-from zwave_js_server.model.firmware import FirmwareUpdateInfo
+from zwave_js_server.model.firmware import (
+    FirmwareUpdateFinished,
+    FirmwareUpdateInfo,
+    FirmwareUpdateProgress,
+    FirmwareUpdateStatus,
+)
 from zwave_js_server.model.node import Node as ZwaveNode
 
 from homeassistant.components.update import UpdateDeviceClass, UpdateEntity
@@ -63,7 +69,9 @@ class ZWaveNodeFirmwareUpdate(UpdateEntity):
     _attr_entity_category = EntityCategory.CONFIG
     _attr_device_class = UpdateDeviceClass.FIRMWARE
     _attr_supported_features = (
-        UpdateEntityFeature.INSTALL | UpdateEntityFeature.RELEASE_NOTES
+        UpdateEntityFeature.INSTALL
+        | UpdateEntityFeature.RELEASE_NOTES
+        | UpdateEntityFeature.PROGRESS
     )
     _attr_has_entity_name = True
     _attr_should_poll = False
@@ -78,6 +86,11 @@ class ZWaveNodeFirmwareUpdate(UpdateEntity):
         self._latest_version_firmware: FirmwareUpdateInfo | None = None
         self._status_unsub: Callable[[], None] | None = None
         self._poll_unsub: Callable[[], None] | None = None
+        self._progress_unsub: Callable[[], None] | None = None
+        self._finished_unsub: Callable[[], None] | None = None
+        self._num_files_installed: int = 0
+        self._finished_event = asyncio.Event()
+        self._finished_status: FirmwareUpdateStatus | None = None
 
         # Entity class attributes
         self._attr_name = "Firmware"
@@ -93,9 +106,59 @@ class ZWaveNodeFirmwareUpdate(UpdateEntity):
         self._status_unsub = None
         self.hass.async_create_task(self._async_update())
 
+    @callback
+    def _update_progress(self, event: dict[str, Any]) -> None:
+        """Update install progress on event."""
+        progress: FirmwareUpdateProgress = event["firmware_update_progress"]
+        if not self._latest_version_firmware:
+            return
+        # We will assume that each file in the firmware update represents an equal
+        # percentage of the overall progress. This is likely not true because each file
+        # may be a different size, but it's the best we can do since we don't know the
+        # total number of fragments across all files.
+        self._attr_in_progress = floor(
+            100
+            * (
+                self._num_files_installed
+                + (progress.sent_fragments / progress.total_fragments)
+            )
+            / len(self._latest_version_firmware.files)
+        )
+        self.async_write_ha_state()
+
+    @callback
+    def _update_finished(self, event: dict[str, Any]) -> None:
+        """Update install progress on event."""
+        finished: FirmwareUpdateFinished = event["firmware_update_finished"]
+        self._finished_status = finished.status
+        self._finished_event.set()
+
+    @callback
+    def _unsub_firmware_events_and_reset_progress(
+        self, write_state: bool = False
+    ) -> None:
+        """Unsubscribe from firmware events and reset update install progress."""
+        if self._progress_unsub:
+            self._progress_unsub()
+            self._progress_unsub = None
+
+        if self._finished_unsub:
+            self._finished_unsub()
+            self._finished_unsub = None
+
+        self._finished_status = None
+        self._finished_event.clear()
+        self._num_files_installed = 0
+        self._attr_in_progress = 0
+        if write_state:
+            self.async_write_ha_state()
+
     async def _async_update(self, _: HomeAssistant | datetime | None = None) -> None:
         """Update the entity."""
         self._poll_unsub = None
+
+        # If device is asleep/dead, wait for it to wake up/become alive before
+        # attempting an update
         for status, event_name in (
             (NodeStatus.ASLEEP, "wake up"),
             (NodeStatus.DEAD, "alive"),
@@ -152,18 +215,50 @@ class ZWaveNodeFirmwareUpdate(UpdateEntity):
         """Install an update."""
         firmware = self._latest_version_firmware
         assert firmware
-        try:
-            for file in firmware.files:
+        self._unsub_firmware_events_and_reset_progress(True)
+
+        self._progress_unsub = self.node.on(
+            "firmware update progress", self._update_progress
+        )
+        self._finished_unsub = self.node.once(
+            "firmware update finished", self._update_finished
+        )
+
+        for file in firmware.files:
+            try:
                 await self.driver.controller.async_begin_ota_firmware_update(
                     self.node, file
                 )
-        except BaseZwaveJSServerError as err:
-            raise HomeAssistantError(err) from err
-        else:
-            self._attr_installed_version = self._attr_latest_version = firmware.version
-            self._latest_version_firmware = None
+            except BaseZwaveJSServerError as err:
+                self._unsub_firmware_events_and_reset_progress()
+                raise HomeAssistantError(err) from err
+
+            # We need to block until we receive the `firmware update finished` event
+            await self._finished_event.wait()
+            assert self._finished_status is not None
+
+            # If status is not OK, we should throw an error to let the user know
+            if self._finished_status not in (
+                FirmwareUpdateStatus.OK_NO_RESTART,
+                FirmwareUpdateStatus.OK_RESTART_PENDING,
+                FirmwareUpdateStatus.OK_WAITING_FOR_ACTIVATION,
+            ):
+                status = self._finished_status
+                self._unsub_firmware_events_and_reset_progress()
+                raise HomeAssistantError(status.name.replace("_", " ").title())
+
+            # If we get here, the firmware installation was successful and we need to
+            # update progress accordingly
+            self._num_files_installed += 1
+            self._attr_in_progress = floor(
+                100 * self._num_files_installed / len(firmware.files)
+            )
             self.async_write_ha_state()
 
+        self._attr_installed_version = self._attr_latest_version = firmware.version
+        self._latest_version_firmware = None
+        self._unsub_firmware_events_and_reset_progress()
+
     async def async_poll_value(self, _: bool) -> None:
         """Poll a value."""
         LOGGER.error(
@@ -189,6 +284,14 @@ class ZWaveNodeFirmwareUpdate(UpdateEntity):
             )
         )
 
+        self.async_on_remove(
+            async_dispatcher_connect(
+                self.hass,
+                f"{DOMAIN}_{self._base_unique_id}_remove_entity_on_ready_node",
+                self.async_remove,
+            )
+        )
+
         self.async_on_remove(async_at_start(self.hass, self._async_update))
 
     async def async_will_remove_from_hass(self) -> None:
@@ -200,3 +303,5 @@ class ZWaveNodeFirmwareUpdate(UpdateEntity):
         if self._poll_unsub:
             self._poll_unsub()
             self._poll_unsub = None
+
+        self._unsub_firmware_events_and_reset_progress()
